# coding:utf-8
"""Some function to generate code."""
import random
import time
import string
import hmac

pool = string.ascii_letters + string.digits


def generate_id():
    stamp = hex(int(time.time()))[2:]
    rand = ''.join(random.choice(pool) for _ in range(8))
    return rand + stamp


def encode_passwd(mixin, passwd):
    return hmac.new(mixin.encode(), passwd.encode()).hexdigest()



# coding:utf-8
"""Some Verify Function."""
import re


class Patterns():

    def __init__(self):
        self.email = re.compile(r'^([\w\-.]+)@([\w-]+)(\.([\w-]+))+$')
        self.password = re.compile(
            r'^[0-9A-Za-z`~!@#$%^&*()_+\-=\{\}\[\]:;"\'<>,.\\|?/]{6,24}$')

    def check(self, pattern_name, target):
        if not hasattr(self, pattern_name):
            return False
        pattern = self.__getattribute__(pattern_name)
        return re.match(pattern, target)



# coding:utf-8
from functools import wraps
import traceback
import enum
from datetime import datetime, timezone
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Query, Session
from motor import MotorGridFSBucket, motor_tornado as motor
from pymongo import MongoClient

from .utils.logger import dump_in, dump_out, dump_error
from .config import CONFIG as O_O


class MongoBase():
    """Mongo Client Set."""

    __collection_name__ = None
    __alias__ = None

    def __init__(self, client):
        self.client = client

    @staticmethod
    def __collection_init__(client):
        """Write some collection initial operation, like set some index."""
        pass


class MongoCollection():

    _instance = None

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(MongoCollection, cls).__new__(cls)
            cls._instance.init()
        return cls._instance

    def init(self):
        T_T = O_O.database.mongo
        self._mongo_client = MongoClient(T_T.client).__getattr__(T_T.db)
        self._motor_client = motor.MotorClient(T_T.client).__getattr__(T_T.db)
        self._collection_dict = {(scls.__alias__ or scls.__collection_name__):
                                 scls for scls in MongoBase.__subclasses__()}

        if T_T.run_init:
            for scls in MongoBase.__subclasses__():
                scls.__collection_init__(self._mongo_client)


class MongoFactory():
    """Mongo Client Set."""

    def __init__(self, sync=False):
        mongo_collection = MongoCollection()
        self._collection_dict = mongo_collection._collection_dict
        self._mongo_client = mongo_collection._mongo_client
        self._motor_client = mongo_collection._motor_client

        if sync:
            self._client = self._mongo_client
        else:
            self._client = self._motor_client

    def __getattr__(self, name):
        if name in self._collection_dict:
            return self._collection_dict[name](self._client)
        else:
            return self.__getattribute__(name)


class SessionMaker:

    _instance = None

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(SessionMaker, cls).__new__(cls)
        return cls._instance

    def __init__(self):
        self.new_engine = create_engine(
            O_O.database.mysql,
            echo=False,
            pool_recycle=1000,
            encoding='utf-8',
            isolation_level='REPEATABLE_READ')
        self.session_maker = sessionmaker(bind=self.new_engine)

    def create(self):
        return self.session_maker()


class BaseMapper:
    """Base Mapper."""

    __alias__ = None

    def __init__(self, session):
        if not isinstance(session, Session):
            raise ValueError(
                "session is not instance of sqlalchemy.orm.session.Session.")
        self.session = session


class DaoContext:

    def __init__(self, session, mapper):
        self.session = session
        self.mapper = mapper
        self.mongo = MongoFactory(sync=True)

    def get_mapper(self):
        return self.mapper(self.session)

    def commit(self):
        self.session.commit()

    def dict_of(self, entity, *options, **alias):
        if entity is None:
            return dict()

        res = dict()
        for key in options:
            alias[key] = None

        for key in entity.__dict__:
            if not alias or key in alias:
                if not key.startswith('_'):
                    if isinstance(alias.get(key), str):
                        real_key = alias[key]
                    else:
                        real_key = key
                    if isinstance(entity.__dict__[key], enum.Enum):
                        res[real_key] = entity.__dict__[key].name
                    elif isinstance(entity.__dict__[key], datetime):
                        t = entity.__dict__[key]
                        t = t.replace(tzinfo=timezone.utc)
                        res[real_key] = int(t.timestamp())
                    else:
                        res[real_key] = entity.__dict__[key]

        return res


def transaction(function, mappers, rollback=True):
    """Wrap a handle shell to a query function."""

    @wraps(function)
    def wrapper(*args, **kwargs):
        """Function that wrapped."""
        session = SessionMaker().create()
        if mappers:
            mapper_dict = {(scls.__alias__ or scls.__name__): scls
                           for scls in BaseMapper.__subclasses__()}
            parents = tuple(mapper_dict[m] for m in mappers if m in mapper_dict)
            mapper = type('Mapper', parents, {})
        else:
            mapper = BaseMapper

        context = DaoContext(session, mapper)

        try:
            return function(context, *args, **kwargs)
        except Exception as e:
            session.rollback()
            raise e
        finally:
            session.close()

    return wrapper


# coding:utf-8
import time
import requests

import gevent

from tarpit.task import service
from tarpit.utils import encode_passwd
from tarpit.config import CONFIG as O_O


@service(mappers=['user', 'user_wallet'])
def create_account(self, name, phone, password, role, avatar):
    mapper = self.get_mapper()

    password = encode_passwd(O_O.server.pass_mixin, password)
    user = mapper.insert_user(name, phone, password, role, avatar)
    mapper.insert_wallet(user.user_id)

    self.commit()

    return dict(user_id=user.user_id)


@service(mappers=['user'])
def delete_account(self, user_id):

    mapper = self.get_mapper()
    user = mapper.update_user_deleted(user_id, 1)
    self.commit()

    return dict(user_id=user.user_id)


@service(mappers=['user'])
def get_account_info(self, user_id):
    mapper = self.get_mapper()
    user_info, balance = mapper.query_user_by_id(user_id)
    option = ['user_id', 'name', 'phone', 'avatar', 'role', 'banned', 'deleted']
    return dict(self.dict_of(user_info, *option), balance=balance)



from tornado.web import HTTPError

__all__ = [
    'MissingArgumentError',
    'TokenExpiredError',
    'ArgumentTypeError',
    'InternalServerError',
    'AuthenticationError',
    'AuthorizationError',
    'ParseJSONError',
    'HTTPError',
    'ServiceError',
]


class MissingArgumentError(HTTPError):
    """Missing Argument Error."""

    def __init__(self, arg_name):
        super(MissingArgumentError, self).__init__(
            400, f'Missing argument "{arg_name}"')
        self.arg_name = arg_name


class TokenExpiredError(HTTPError):
    """Token Expired Error."""

    def __init__(self):
        super(TokenExpiredError, self).__init__(401, 'Token Expired')


class ArgumentTypeError(HTTPError):
    """Argument Type Error."""

    def __init__(self, arg_name, type_name):
        super(ArgumentTypeError, self).__init__(
            400, f'Argument "{arg_name}" should be {type_name}')


class InternalServerError(HTTPError):
    """Task Error."""

    def __init__(self, task_name, error_msg):
        super(InternalServerError, self).__init__(
            500, f'Error occured when excute task "{task_name}".\n{error_msg}')


class AuthenticationError(HTTPError):
    """Authentication Error."""

    def __init__(self, msg):
        super(AuthenticationError, self).__init__(401, msg)


class AuthorizationError(HTTPError):
    """Authorization Error."""

    def __init__(self, arg_name):
        super(AuthorizationError, self).__init__(
            401, f'User\'s "{arg_name}" not satisfied.')


class ParseJSONError(HTTPError):
    """Exception raised by `BaseHandler.parse_json`.

    This is a subclass of `HTTPError`, so if it is uncaught a 400 response
    code will be used instead of 500 (and a stack trace will not be logged).
    """

    def __init__(self, doc):
        super(ParseJSONError, self).__init__(400, f'ParseJSONError "{doc}"')


class ServiceError(Exception):

    def __init__(self, code, desc):
        self.code = code
        self.desc = desc



# coding:utf-8
"""Worker Module."""
import sys

from tarpit.task import CeleryConfig
from tarpit.config import _ENV as env

if __name__ == 'worker':
    queue = None
    cc = CeleryConfig()
    if 'worker' in sys.argv[:3]:
        for i, arg in enumerate(sys.argv):
            if arg.startswith(env) and sys.argv[i - 1] == '-Q':
                queue = arg.split(':')[1]
                break

        if queue in cc.queue_list:
            __import__(cc.queue_list[queue]['module'])
        else:
            exit(f'Unknown queue name: "{queue}"')



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages

setup(
    name='tarpit',
    version='0.0.11',
    author='plank',
    author_email='sieglive@gmail.com',
    url='https://lazor.cn',
    description='一个快速构建服务后台的库，可以快速创建简单后台。',
    license="GPLv3+",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'alembic>=1.0.7',
        'celery>=4.2.1',
        'gevent>=1.4.0',
        'motor>=2.0.0',
        'PyJWT>=1.7.1',
        'pymongo>=3.7.2',
        'PyMySQL>=0.9.3',
        'PyYAML>=5.1',
        'redis>=3.2.0',
        'requests>=2.21.0',
        'SQLAlchemy>=1.2.18',
        'tornado>=5.1.1',
        'cryptography>=2.6.1',
    ],
    entry_points={'console_scripts': ['tarpit=tarpit:main']},
)



class RequestParameter:

    __param_name__ = None
    __param_type__ = None
    __param_subparam__ = None
    __param_optional__ = True
    __param_default__ = None

    def __init__(self, name):
        self.__param_name__ = name

    def __rshift__(self, attr_type):
        if self.__param_type__ is not None:
            raise ValueError('Type of Parameter already set.')
        if not isinstance(attr_type, (type, list, set)):
            raise ValueError(
                f'Type of Parameter must be a instance of "type" "list" or "set", not "{type(attr_type).__name__}"'
            )
        if isinstance(attr_type, type):
            self.__param_type__ = attr_type
        elif isinstance(attr_type, (list, set)):
            print(attr_type)
            self.__param_type__ = type(attr_type)
            self.__param_subparam__ = attr_type
            self.__param_default__ = type(attr_type)()

        return self

    @property
    def optional(self):
        return self.__param_optional__

    @property
    def default(self):
        return self.__param_default__

    @property
    def type(self):
        return self.__param_type__

    @property
    def name(self):
        return self.__param_name__

    @property
    def subparam(self):
        return self.__param_subparam__


class P(RequestParameter):
    __param_optional__ = True

    def __ge__(self, value):
        self.__param_default__ = value
        return self


class M(RequestParameter):
    __param_optional__ = False

    def __ge__(self, value):
        raise ValueError('Only optional parameter can set default value.')